# © 2024 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import _, api, Command, fields, models
from odoo.exceptions import ValidationError


class FinancialContractGuarantee(models.Model):
    _name = "financial.contract.guarantee"
    _inherit = [
        "financial.contract",
        "mail.thread",
        "mail.activity.mixin",
    ]
    _description = "Contrat de garantie"
    _check_company_auto = True
    _rec_names_search = [
        "name",
        "number",
        "partner_id.name",
        "partner_id.member_number",
    ]
    _order = "create_date desc"

    product_id = fields.Many2one(
        comodel_name="financial.product.template.guarantee",
        string="Gamme",
        check_company=True,
        required=True,
    )
    suspensive_condition_ids = fields.One2many(
        comodel_name="financial.contract.guarantee.suspensive.condition",
        inverse_name="contract_id",
        compute="_compute_suspensive_condition_ids",
        store=True,
        readonly=False,
    )
    # TODO: voir pour la gestion de ce champ avec le module contract external
    external_loan_id = fields.Many2one(
        comodel_name="financial.contract.external",
        string="Prêt lié",
        domain="[('partner_id', '=', partner_id), ('tool', '=', 'loan')]",
    )
    loan_contract_number = fields.Char(
        string="Numéro de contrat de prêt",
        related="external_loan_id.number",
        store=True,
    )
    guarantee_rate = fields.Float("Quotité garantie", tracking=1)
    initial_guarantee_amount = fields.Monetary(
        string="Montant initial garanti",
        compute="_compute_initial_guarantee_amount",
        store=True,
    )
    guarantee_amount = fields.Monetary(
        compute="_compute_guarantee_amount",
        string="En cours de garantie",
        store=True,
        tracking=1,
    )
    is_counter_guarantee = fields.Boolean("Contre Garantie", default=False)
    counter_guarantee_rate = fields.Float("Quotité contre garantie")
    counter_guarantee_partner_id = fields.Many2one(
        comodel_name="financial.contract.guarantee.partner",
        string="Partenaire"
    )
    final_risk_guarantee = fields.Monetary(
        string="Garantie risque final",
        compute="_compute_final_risk_guarantee",
        store=True,
        tracking=1,
    )
    fmg_amount = fields.Monetary(
        string="FMG calculé",
        compute="_compute_fmg_amount",
        store=True,
    )
    fmg_paid = fields.Monetary(
        string="FMG payé",
        compute="_compute_fmg_paid",
        store=True,
    )
    fmg_ids = fields.One2many(
        comodel_name="mutual.guarantee.fund.line",
        inverse_name="guarantee_id",
        string="FMG",
    )
    social_share_number = fields.Integer(
        string="Nbre parts sociales",
        compute="_compute_social_share",
        store=True
    )
    social_share_amount = fields.Monetary(
        string="Montant parts sociales calculé",
        compute="_compute_social_share",
        store=True
    )
    social_share_ids = fields.One2many(
        comodel_name="company.share.line",
        inverse_name="guarantee_id",
        string="FMG",
    )
    social_share_paid = fields.Monetary(
        string="Montant parts sociales payé",
        compute="_compute_social_share_paid",
        store=True
    )
    line_ids = fields.One2many(
        comodel_name="financial.contract.guarantee.line",
        inverse_name="guarantee_id",
        string="Lignes de solde",
    )

    # Loan Data
    payment_date = fields.Date("Date de versement")
    amount_initial = fields.Monetary(
        string="Montant initial",
        compute="_compute_amount_initial",
        store=True,
        readonly=False,
    )
    amount_received = fields.Monetary("Montant reçu")
    end_date = fields.Date("Date de fin de prêt")
    remaining_capital = fields.Monetary("Capital restant dû", tracking=1)
    loan_duration = fields.Integer(
        string="Durée du crédit (mois)",
        compute="_compute_loan_duration",
        store=True,
        readonly=False,
    )

    # Company Data
    segment_code = fields.Char("Code Segment")
    bdf_scoring = fields.Char(
        string="Cotation BDF",
        compute="_compute_guarantee_data",
        store=True,
        readonly=False,
        tracking=2,
    )
    bdf_date = fields.Char("Date Cotation BDF")
    mcdo_scoring = fields.Char("Cotation MacDonough")
    mcdo_date = fields.Date("Date Cotation MacDonough")
    branch_code = fields.Char("Code branche")

    # Invoice Data
    commission_amount = fields.Monetary(
        string="Montant de commission",
        compute="_compute_commission_amount",
        store=True,
    )

    # ------------------------------------------------------
    # Constrains functions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.depends("external_loan_id", "external_loan_id.amount")
    def _compute_amount_initial(self):
        for guarantee in self:
            if guarantee.external_loan_id:
                guarantee.amount_initial = guarantee.external_loan_id.amount

    @api.depends("external_loan_id", "external_loan_id.duration")
    def _compute_loan_duration(self):
        for guarantee in self:
            if guarantee.external_loan_id:
                guarantee.loan_duration = guarantee.external_loan_id.duration

    @api.depends(
        "line_ids",
        "line_ids.bdf_scoring",
    )
    def _compute_guarantee_data(self):
        for guarantee in self:
            last_line = guarantee.line_ids.sorted("line_date desc")
            guarantee.update({"bdf_scoring": last_line.bdf_scoring})

    @api.depends(
        "line_ids",
        "line_ids.commission_amount",
    )
    def _compute_commission_amount(self):
        for guarantee in self:
            guarantee.commission_amount = sum(
                guarantee.line_ids.mapped("commission_amount")
            )

    @api.depends("guarantee_amount", "is_counter_guarantee", "counter_guarantee_rate")
    def _compute_final_risk_guarantee(self):
        for guarantee in self:
            if not guarantee.is_counter_guarantee:
                guarantee.final_risk_guarantee = guarantee.guarantee_amount
            else:
                guarantee.final_risk_guarantee = (1 - guarantee.counter_guarantee_rate) * guarantee.guarantee_amount

    @api.depends("remaining_capital", "guarantee_rate")
    def _compute_guarantee_amount(self):
        for guarantee in self:
            guarantee.guarantee_amount = guarantee.remaining_capital * guarantee.guarantee_rate

    @api.depends("amount_initial", "guarantee_rate")
    def _compute_initial_guarantee_amount(self):
        for guarantee in self:
            guarantee.initial_guarantee_amount = guarantee.amount_initial * guarantee.guarantee_rate

    @api.depends("initial_guarantee_amount")
    def _compute_fmg_amount(self):
        fmg_rate = self.company_id.fmg_rate
        if not fmg_rate or fmg_rate == 0.0:
            raise ValidationError(_("Le taux de FMG n'est pas configuré."))
        for guarantee in self:
            guarantee.fmg_amount = fmg_rate * guarantee.initial_guarantee_amount

    @api.depends("fmg_ids", "fmg_ids.amount")
    def _compute_fmg_paid(self):
        for guarantee in self:
            guarantee.fmg_paid = sum(guarantee.fmg_ids.mapped("amount"))

    @api.depends("initial_guarantee_amount", "partner_id")
    def _compute_social_share(self):
        """
        Calcule nombre et montant des parts sociales en fonction :
        - du taux défini sur la société
        - du montant max défini sur la société
        - du prix unitaire de la part défini sur la société
        - des prises de parts existantes sur la société
        :return: affecte les valeurs de montant et nombre de parts
        """
        social_share_rate = self.company_id.social_share_rate
        social_share_amount_max = self.company_id.social_share_amount_max
        share_unit_price = self.company_id.share_unit_price
        for guarantee in self:
            max_amount = social_share_amount_max - guarantee.partner_id.company_share_total
            rate_amount = social_share_rate * guarantee.initial_guarantee_amount
            final_amount = rate_amount if rate_amount <= max_amount else max_amount
            guarantee.social_share_number = (final_amount // share_unit_price)
            guarantee.social_share_amount = guarantee.social_share_number * share_unit_price

    @api.depends("social_share_ids", "social_share_ids.share_total_amount")
    def _compute_social_share_paid(self):
        for guarantee in self:
            guarantee.social_share_paid = sum(guarantee.social_share_ids.mapped("share_total_amount"))

    @api.depends("product_id")
    def _compute_suspensive_condition_ids(self):
        for contract in self:
            condition_to_create = []
            if contract.product_id:
                for condition in self.product_id.suspensive_condition_ids:
                    condition_to_create.append(
                        Command.create(
                            {
                                "condition_id": condition.id,
                                "condition_comment": condition.condition_comment,
                            },
                        )
                    )
                if condition_to_create:
                    contract.suspensive_condition_ids.unlink()
                    contract.suspensive_condition_ids = condition_to_create

    # ------------------------------------------------------
    # Onchange
    # ------------------------------------------------------
    @api.onchange("product_id")
    def _onchange_product_id(self):
        self._set_product_values()

    @api.onchange("is_counter_guarantee")
    def _onchange_is_counter_guarantee(self):
        if not self.is_counter_guarantee:
            self.counter_guarantee_rate = 0.0
            self.counter_guarantee_partner_id = False

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD (Override ORM)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business functions
    # ------------------------------------------------------
    def _set_product_values(self):
        if self.product_id:
            self.guarantee_rate = self.product_id.guarantee_rate
